<?php

namespace Drupal\comment_fragment\EventSubscriber;

use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Redirects comments canonical URLs to their containing entities.
 *
 * @package Drupal\comment_fragment\EventSubscriber
 */
class CommentFragmentSubscriber implements EventSubscriberInterface {

  /**
   * Route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructor of the class.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   Route match service.
   */
  public function __construct(RouteMatchInterface $route_match) {
    $this->routeMatch = $route_match;
  }

  /**
   * Registers the methods in this class that should be listeners.
   *
   * @return array
   *   An array of event listener definitions.
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onRequest'];
    return $events;
  }

  /**
   * Manipulates the request object.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The Event to process.
   */
  public function onRequest(RequestEvent $event) {
    $route_name = $this->routeMatch->getRouteName();
    if ($route_name == 'entity.comment.canonical') {
      $comment = $event->getRequest()->attributes->get('comment', NULL);
      if ($comment) {
        $url = comment_fragment_uri($comment);
        $response = new RedirectResponse($url->setAbsolute()->toString());
        $event->setResponse($response);
      }
    }
  }

}
